@@ -3,14 +3,15 @@ setup_training <- function(d, outcome, model_class, models, metric, positive_cla
 
   # Get recipe and remove columns to be ignored in training
   recipe <- attr(d, "recipe")
-  if (!is.null(recipe)) {
-    outcome_chr <- rlang::quo_name(outcome)
-    if (outcome_chr %in% attr(recipe, "ignored_columns"))
-      stop("You specified ", outcome_chr, " as your outcome variable, but ",
-           "the recipe you used to prep your data says to ignore it. ",
-           "Did you forget to specify `outcome = ` in prep_data?")
-    d <- remove_ignored(d, recipe)
-  }
+  if (is.null(recipe))
+    stop("data must be prepped before training models. If you want to prep ",
+         "data yourself (not recommended), use `prep_data(..., no_prep = TRUE)`")
+  outcome_chr <- rlang::quo_name(outcome)
+  if (outcome_chr %in% attr(recipe, "ignored_columns"))
+    stop("You specified ", outcome_chr, " as your outcome variable, but ",
+         "the recipe you used to prep your data says to ignore it. ",
+         "Did you forget to specify `outcome = ` in prep_data?")
+  d <- remove_ignored(d, recipe)
 
   # Check outcome provided, agrees with outcome in prep_data, present in d
   outcome <- check_outcome(outcome, names(d), recipe)
@@ -34,8 +35,9 @@ setup_training <- function(d, outcome, model_class, models, metric, positive_cla
     dplyr::filter(!is_numeric) %>%
     dplyr::pull(variable)
   if (length(non_numerics))
-    stop("All predictors must be numeric, but the variables are not numeric. Consider using ",
-         "prep_data to get data ready for model training: ", list_variables(non_numerics))
+    stop("All predictors must be numeric, but the following variables are not ",
+         "numeric. Consider using prep_data to get data ready for model ",
+         "training: ", list_variables(non_numerics))
 
   if (missing(models)) {
     models <- get_supported_models()
@@ -43,22 +45,26 @@ setup_training <- function(d, outcome, model_class, models, metric, positive_cla
     # Make `models` case insensitive
     models <- tolower(models)
   }
-
   # Make sure outcome's class works with model_class, or infer it
-  model_class <- set_model_class(model_class, class(dplyr::pull(d, !!outcome)), outcome_chr)
-
+  model_class <- set_model_class(model_class,
+                                 dplyr::pull(d, !!outcome),
+                                 outcome_chr)
+  outcome_tab <- table(d[[outcome_chr]])
   if (model_class == "classification") {
     if (missing(positive_class))
       positive_class <- NULL
-    # Some algorithms need the response to be factor instead of char or lgl
-    # Get rid of unused levels if they're present
     d[[outcome_chr]] <-
       d[[outcome_chr]] %>%
+      # Some algorithms need the response to be factor instead of char or lgl
       as.factor() %>%
+      # Get rid of unused levels if they're present
       droplevels() %>%
-      set_outcome_class(positive_class)
+      # Choose positive class and set it to the factor reference level
+      set_outcome_class(positive_class = positive_class,
+                        original_classes = names(attributes(recipe)$factor_levels[[outcome_chr]]))
     # Make sure there can be at least one instance of outcome in each fold
-    outcome_tab <- table(d[[outcome_chr]])
+  }
+  if (model_class == "classification") {
     if (min(outcome_tab) < n_folds)
       stop("There must be at least one instance of each outcome class ",
            "for each cross validation fold. Observed frequencies in d:\n",
@@ -66,10 +72,17 @@ setup_training <- function(d, outcome, model_class, models, metric, positive_cla
            "\nYou could try turning n_folds down from its current value of ", n_folds,
            ", but it's hard to train a good model with few observations of an outcome.")
   }
+  if (model_class == "multiclass")
+    if (max(outcome_tab) / nrow(d) <= 0.05)
+      warning("Your outcome variable has categories that are sparsely ",
+              "distributed. It may be hard for the model to correctly predict ",
+              "them.")
 
   # Choose metric if not provided
   if (missing(metric))
     metric <- set_default_metric(model_class)
+  else
+    metric <- check_metric(model_class, metric)
 
   # Make sure models are supported
   models <- check_models(models)
@@ -88,8 +101,14 @@ setup_training <- function(d, outcome, model_class, models, metric, positive_cla
 }
 
 check_outcome <- function(outcome, d_names, recipe) {
-  if (rlang::quo_is_missing(outcome))
-    stop("You must provide an outcome variable to tune_models and flash_models.")
+  if (rlang::quo_is_missing(outcome)){
+    if (is.null(recipe))
+      stop("Your data is not prepared. Either provide provide an outcome ",
+           "variable to tune_models and flash_models, or prepare your data in ",
+           "prep_data")
+    else
+      outcome <- recipe$var_info$variable[recipe$var_info$role == "outcome"]
+  }
   outcome_chr <- rlang::quo_name(outcome)
   if (!outcome_chr %in% d_names)
     stop(outcome_chr, " isn't a column in d.")
@@ -102,10 +121,15 @@ check_outcome <- function(outcome, d_names, recipe) {
   return(outcome)
 }
 
-set_outcome_class <- function(vec, positive_class) {
+set_outcome_class <- function(vec, positive_class, original_classes) {
   if (length(levels(vec)) != 2)
     stop(paste0("The outcome variable must have two levels for classification, ",
                 "but this has ", length(levels(vec)), ": ", paste(levels(vec), collapse = ", ")))
+  if (!all(original_classes %in% vec))
+    stop("It looks like outcome levels have changed between data prep and ",
+         "model training. Change them before prep_data instead.\nPre-prep ",
+         "values: ", list_variables(original_classes), "\nCurrent values: ",
+         list_variables(unique(vec)))
   if (is.null(positive_class)) {
     positive_class <-
       if ("Y" %in% levels(vec)) {
@@ -119,7 +143,7 @@ set_outcome_class <- function(vec, positive_class) {
   if (!positive_class %in% levels(vec))
     stop("positive_class, ", positive_class, ", not found in the outcome column. ",
          "Outcome has values ", list_variables(levels(vec)) )
-  vec <- stats::relevel(vec, setdiff(levels(vec), positive_class))
+  vec <- stats::relevel(vec, positive_class)
   return(vec)
 }
 
@@ -136,7 +160,9 @@ remove_ignored <- function(d, recipe) {
   return(d)
 }
 
-set_model_class <- function(model_class, outcome_class, outcome_chr) {
+set_model_class <- function(model_class, outcome, outcome_chr) {
+  n_outcomes <- dplyr::n_distinct(outcome)
+  outcome_class <- class(outcome)
   looks_categorical <- outcome_class %in% c("character", "factor")
   looks_numeric <- outcome_class %in% c("integer", "numeric")
   if (!looks_categorical && !looks_numeric) {
@@ -146,8 +172,13 @@ set_model_class <- function(model_class, outcome_class, outcome_chr) {
   } else if (missing(model_class)) {
     # Need to infer model_class
     if (looks_categorical) {
-      mes <- paste0(outcome_chr, " looks categorical, so training classification algorithms.")
-      model_class <- "classification"
+      if (n_outcomes > 2) {
+        mes <- paste0(outcome_chr, " looks multiclass, so training multiclass algorithms.")
+        model_class <- "multiclass"
+      } else {
+        mes <- paste0(outcome_chr, " looks categorical, so training classification algorithms.")
+        model_class <- "classification"
+      }
     } else {
       mes <- paste0(outcome_chr, " looks numeric, so training regression algorithms.")
       model_class <- "regression"
@@ -163,9 +194,12 @@ set_model_class <- function(model_class, outcome_class, outcome_chr) {
            ". You supplied this unsupported class: ", model_class)
     if (looks_categorical && model_class == "regression") {
       stop(outcome_chr, " looks categorical but you're trying to train a regression model.")
-    } else if (looks_numeric && model_class == "classification") {
+    } else if (looks_numeric && model_class %in% c("classification", "multiclass")) {
       stop(outcome_chr, " looks numeric but you're trying to train a classification ",
            "model. If that's what you want convert it explicitly with as.factor().")
+    } else if (looks_categorical && model_class == "classification" && n_outcomes > 2) {
+      stop(outcome_chr, " looks multiclass but you're trying to train a 2-class,
+           classification model. Use model_class = 'Multiclass'")
     }
   }
   return(model_class)
@@ -183,13 +217,37 @@ check_models <- function(models) {
   return(models)
 }
 
+check_metric <- function(model_class, metric) {
+  if (is.na(metric)) {
+    metric <- set_default_metric(model_class)
+    warning("The given metric is NA, evaluating models with ", metric,
+            " instead")
+  } else if ( (model_class == "regression" &&
+               !(metric %in% c("MAE", "RMSE", "Rsquared"))) ||
+              (model_class == "classification" &&
+               !(metric %in% c("ROC", "PR"))) ||
+              (model_class == "multiclass" &&
+               !(metric %in% c("Accuracy", "Kappa")))) {
+      new_metric <- set_default_metric(model_class)
+      warning("Healthcareai does not support ", metric,
+              ", evaluating models with ", new_metric, " instead")
+      metric <- new_metric
+  } else if (!(model_class %in%
+               c("regression", "classification", "multiclass"))) {
+    stop("Healthcareai does not support ", model_class, " yet.")
+  }
+  return(metric)
+}
+
 set_default_metric <- function(model_class) {
   if (model_class == "regression") {
     return("RMSE")
   } else if (model_class == "classification") {
     return("ROC")
+  } else if (model_class == "multiclass") {
+    return("Accuracy")
   } else {
-    stop("Don't have default metric for model class", model_class)
+    stop("Don't have default metric for model class ", model_class)
   }
 }
 
@@ -230,8 +288,8 @@ check_training_time <- function(ddim, hpdim, n_folds) {
   ddim[2] <- ddim[2] - 1L
   ncells <- prod(ddim)
   n_models <-
-      paste0(n_folds * hpdim, " ", names(hpdim), "'s") %>%
-      list_variables()
+    paste0(n_folds * hpdim, " ", names(hpdim), "'s") %>%
+    list_variables()
   mes <- paste(
     "\nAfter data processing, models are being trained on", format(ddim[2], big.mark = ","),
     "features with", format(ddim[1], big.mark = ","), "observations.\nBased on n_folds =",
