@@ -2,132 +2,48 @@ context("model_list tests setup")
 
 # Setup ------------------------------------------------------------------------
 set.seed(257056)
-data(mtcars)
-nadd <- 50
-mtcars <- rbind(mtcars,
-                data.frame(mpg = rnorm(nadd, 25, 5),
-                           cyl = sample(c(4, 6, 8), nadd, TRUE),
-                           disp = rnorm(nadd, 200, 50),
-                           hp = rnorm(nadd, 150, 20),
-                           drat = rexp(nadd, .5),
-                           wt = rnorm(nadd, 4, .5),
-                           qsec = rnorm(nadd, 18, 2),
-                           vs = sample(0:1, nadd, TRUE),
-                           am = sample(0:1, nadd, TRUE),
-                           gear = sample(3:5, nadd, TRUE),
-                           carb = rpois(nadd, 4)))
-dreg <- prep_data(mtcars, outcome = mpg)
-dcla <- prep_data(mtcars, outcome = am)
-suppressWarnings({
-  rf <- caret::train(x = dplyr::select(dreg, -mpg),
-                     y = dreg$mpg,
-                     method = "ranger",
-                     tuneLength = 2,
-                     trControl = caret::trainControl(savePredictions = "final")
-  )
-  xg <- caret::train(x = dplyr::select(dreg, -mpg),
-                     y = dreg$mpg,
-                     method = "xgbTree",
-                     tuneLength = 2,
-                     trControl = caret::trainControl(savePredictions = "final")
-  )
-  gl <- caret::train(x = dplyr::select(dreg, -mpg),
-                     y = dreg$mpg,
-                     method = "glmnet",
-                     tuneLength = 2,
-                     trControl = caret::trainControl(savePredictions = "final")
-  )
-})
+short <- dplyr::sample_n(na.omit(pima_diabetes), 50)
+dreg <- prep_data(short, outcome = pedigree)
+dcla <- prep_data(short, outcome = diabetes)
+dmul <- prep_data(dplyr::sample_n(iris, 100), outcome = Species)
+
 # Implicit test that warning not issued for missing resampled performance metrics:
-r_models <- tune_models(dreg, mpg, n_folds = 2, tune_depth = 2)
-c_models <- tune_models(dcla, am, n_folds = 2, tune_depth = 2,
-                        model_name = "great_name")
-c_pr <- tune_models(dcla, am, metric = "PR", n_folds = 2, tune_depth = 2)
-single_model_as <- as.model_list(rf)
-single_model_tune <- tune_models(dcla, am, models = "rf")
-double_model_as <- as.model_list(rf, xg)
-r_flash <- flash_models(dreg, mpg)
-c_flash <- flash_models(dcla, am)
-unprepped_flash <- flash_models(mtcars, mpg, models = "glm")
-ods <- list(
-  prepped = attr(c_models, "original_data_str"),
-  unprepped = attr(unprepped_flash, "original_data_str"),
-  as = attr(single_model_as, "original_data_str")
-)
+r_models <- tune_models(dreg, pedigree, tune_depth = 2, n_folds = 3, models = c("rf", "glm"))
+c_models <- tune_models(dcla, diabetes, tune_depth = 2, n_folds = 3, model_name = "great_name", models = "xgb")
+m_models <- tune_models(d = dmul, outcome = Species, tune_depth = 2, n_folds = 3, models = "rf")
+c_pr <- flash_models(dcla, diabetes, metric = "PR", n_folds = 2, models = "xgb")
+single_model_tune <- tune_models(dcla, diabetes, models = "xgb", n_folds = 2, tune_depth = 2)
+r_flash <- flash_models(dreg, pedigree, n_folds = 2, models = "xgb")
+c_flash <- flash_models(dcla, diabetes, n_folds = 2, models = "xgb")
+m_flash <- flash_models(dmul, Species, n_folds = 2, models = "xgb")
+ods <- attr(c_models, "original_data_str")
 
 context("Checking model_list constructors") # ----------------------------------
 
-test_that("as.model_list works same with different argument specs", {
-  expect_equivalent(as.model_list(rf),
-                    as.model_list(listed_models = list(rf)))
-  expect_equivalent(as.model_list(rf),
-                    as.model_list(rf, model_class = "regression"))
-})
-
 test_that("as.model_list fails if model_class is unsupported", {
-  expect_error(as.model_list(model_class = "what am i even?"))
+  expect_error(as.model_list(model_class = "what diabetes i even?"))
 })
 
 test_that("as.model_list errors if input isn't a caret model", {
   expect_error(as.model_list(1:5, model_class = "regression"))
   expect_error(as.model_list("HEY"))
-  expect_error(as.model_list(ranger::ranger(mpg ~ ., dreg)))
-})
-
-test_that("as.model_list succeeds with one or more models as input", {
-  expect_s3_class(as.model_list(rf, model_class = "regression"),
-                  "model_list")
-  expect_s3_class(as.model_list(rf, xg), "model_list")
-  expect_s3_class(
-    as.model_list(listed_models = list(rf, xg), model_class = "regression"),
-    "model_list"
-  )
-  expect_s3_class(as.model_list(listed_models = list(rf)), "model_list")
-})
-
-test_that("as.model_list returns correct model names (from modelInfo$label)", {
-  correct_names <- names(r_models)
-  m_list <- structure(list(rf, xg, gl), names = c("rando", "xgb", "lasso"))
-  expect_equal(
-    names(as.model_list(listed_models = m_list)),
-    correct_names
-  )
-  expect_equal(
-    names(as.model_list(rf, xg, gl)),
-    correct_names
-  )
-})
-
-test_that("as.model_list tuned-argument works", {
-  expect_true(attr(as.model_list(rf), "tuned"))
-  expect_false(attr(as.model_list(rf, tuned = FALSE), "tuned"))
+  expect_error(as.model_list(ranger::ranger(pedigree ~ ., dreg)))
 })
 
 test_that("model_lists have original data str as zero-row DF with right names and classes", {
-  purrr::map_lgl(ods, is.data.frame) %>% all() %>% expect_true()
-  purrr::map_lgl(ods, ~ nrow(.x) == 0) %>% all() %>% expect_true()
-  expect_equivalent(ods$prepped, mtcars[0, -which(names(mtcars) == "am")])
-  expect_equivalent(ods$unprepped, mtcars[0, -which(names(mtcars) == "mpg")])
-  expect_equivalent(ods$as, dplyr::select(dreg[0, ], -mpg))
+  expect_s3_class(ods, "data.frame")
+  expect_equal(nrow(ods), 0)
+  expect_equivalent(ods, short[0, -which(names(short) == "diabetes")])
 })
 
 test_that("model_list's original_data_str is the same as predict's return", {
-
-  preds <- purrr::map2(
-    .x = list(c_models, unprepped_flash, single_model_as),
-    .y = list(mtcars, mtcars, mtcars),
-    .f = ~ suppressWarnings( predict(.x, .y)[0, - (1:2)] )
-  )
-  purrr::map2_lgl(preds, ods, all.equal) %>%
-    all() %>%
-    expect_true()
+  preds <- predict(c_models, short)[0, - (1:2)]
+  expect_equal(preds, ods)
 })
 
 test_that("model_lists have r, hcai, and other-package versions as attrs", {
   expected <- c("r_version", "hcai_version", "other_packages")
   expect_true(all(expected %in% names(attributes(r_models)$versions)))
-  expect_true(all(expected %in% names(attributes(single_model_as)$versions)))
-  expect_true(all(expected %in% names(attributes(unprepped_flash)$versions)))
 })
 
 context("Checking model_list generics") # --------------------------------------
@@ -137,9 +53,7 @@ test_that("plot.model_list works on regression_list", {
                c("gg", "ggplot"))
   expect_equal(class(plot.model_list(r_models, print = FALSE)),
                c("gg", "ggplot"))
-  expect_error(plot.model_list(ranger::ranger(mpg ~ ., dreg), print = FALSE),
-               regexp = "model_list")
-  r2 <- tune_models(dreg, mpg, models = "rf", metric = "Rsquared")
+  r2 <- flash_models(dreg, pedigree, models = "rf", n_folds = 3, metric = "Rsquared")
   expect_s3_class(plot(r2, print = FALSE), "gg")
 })
 
@@ -150,8 +64,6 @@ test_that("plot.model_list works on classification_list", {
                c("gg", "ggplot"))
   expect_equal(class(plot(c_pr, print = FALSE)),
                c("gg", "ggplot"))
-  expect_error(plot.model_list(ranger::ranger(am ~ ., dcla), print = FALSE),
-               regexp = "model_list")
 
   # With PR as the metric
   expect_equal(class(plot(c_pr, print = FALSE)),
@@ -171,6 +83,10 @@ test_that("print.model_list works", {
   expect_true(nchar(cprint) > 0)
   expect_true(grepl("classification", cprint, ignore.case = TRUE))
 
+  mprint <- capture_output(m_models, TRUE)
+  expect_true(nchar(mprint) > 0)
+  expect_true(grepl("multiclass", mprint, ignore.case = TRUE))
+
   # With PR as the metric
   cprint <- capture_output(c_pr, TRUE)
   expect_true(nchar(cprint) > 0)
@@ -178,7 +94,7 @@ test_that("print.model_list works", {
 
   # Model name
   rprint <- capture_output(r_models, TRUE)
-  expect_true(grepl("Model Name: mpg", rprint, ignore.case = TRUE))
+  expect_true(grepl("Model Name: pedigree", rprint, ignore.case = TRUE))
   cprint <- capture_output(c_models, TRUE)
   expect_true(grepl("Model Name: great_name", cprint, ignore.case = TRUE))
 })
@@ -209,22 +125,29 @@ context("Checking model_list generics on untuned model_lists") #----------------
 test_that("print.model_list works with untuned_model_lists", {
   expect_warning(flash_r_print <- capture_output(print(r_flash)), NA)
   expect_warning(flash_c_print <- capture_output(print(c_flash)), NA)
+  expect_warning(flash_m_print <- capture_output(print(m_flash)), NA)
   expect_false(grepl("Inf", flash_r_print))
   expect_false(grepl("Inf", flash_c_print))
-  expect_true(grepl("Target: mpg", flash_r_print))
-  expect_true(grepl("Target: am", flash_c_print))
+  expect_false(grepl("Inf", flash_m_print))
+  expect_true(grepl("Target: pedigree", flash_r_print))
+  expect_true(grepl("Target: diabetes", flash_c_print))
+  expect_true(grepl("Target: Species", flash_m_print))
   expect_true(grepl("Models have not been tuned", flash_r_print))
   expect_true(grepl("selected hyperparameter values", flash_c_print))
+  expect_true(grepl("Accuracy", flash_m_print))
 })
 
 test_that("summary.model_list works with untuned_model_lists", {
   expect_warning(flash_r_summary <- capture_output(summary(r_flash)), NA)
   expect_warning(flash_c_summary <- capture_output(summary(c_flash)), NA)
+  expect_warning(flash_m_summary <- capture_output(summary(m_flash)), NA)
   expect_false(grepl("Inf", flash_r_summary))
   expect_false(grepl("Inf", flash_c_summary))
+  expect_false(grepl("Inf", flash_m_summary))
   expect_false(grepl("0 rows", flash_r_summary))
   expect_false(grepl("Best performance:", flash_r_summary))
   expect_true(grepl("Best algorithm:", flash_c_summary))
+  expect_true(grepl("Kappa", flash_m_summary))
 })
 
 test_that("plot.model_list works with message untuned_model_lists", {
@@ -236,28 +159,18 @@ test_that("plot.model_list works with message untuned_model_lists", {
 })
 
 context("Testing model list utilities") # --------------------------------------
-test_that("change_metric_names changes AUC to AUPR", {
+test_that("change_metric_names changes AUC to AUPR and prints", {
   m <- change_metric_names(c_pr)
-
-  expect_true(
-    all(c("AUPR", "Precision", "Recall") %in% names(
-      m$`Random Forest`$results)))
-
-  expect_true(
-    all(c("AUPR", "Precision", "Recall") %in% names(
-      m$`eXtreme Gradient Boosting`$results)))
+  expect_true(all(c("AUPR", "Precision", "Recall") %in% names(m[[1]]$results)))
+  expect_true(all(c("AUPR", "Precision", "Recall") %in% names(m[[1]]$results)))
+  expect_error(capture_output(print(m)), NA)
 })
 
 test_that("change_metric_names changes ROC to AUROC", {
   m <- change_metric_names(c_models)
-
-  expect_true(
-    all(c("AUROC", "Sens", "Spec") %in% names(
-      m$`Random Forest`$results)))
-
-  expect_true(
-    all(c("AUROC", "Sens", "Spec") %in% names(
-      m$`eXtreme Gradient Boosting`$results)))
+  expect_true(all(c("AUROC", "Sens", "Spec") %in% names(m[[1]]$results)))
+  expect_true(all(c("AUROC", "Sens", "Spec") %in% names(m[[1]]$results)))
+  expect_error(capture_output(print(m)), NA)
 })
 
 test_that("Change PR metric doesn't change object class", {
@@ -271,44 +184,41 @@ test_that("model_lists have time model trained attribute", {
   check_timestamp <- function(m) expect_true(lubridate::is.POSIXt(attr(m, "timestamp")))
   check_timestamp(r_models)
   check_timestamp(c_models)
+  check_timestamp(m_models)
   check_timestamp(c_pr)
   check_timestamp(r_flash)
   check_timestamp(c_flash)
+  check_timestamp(m_flash)
 })
 
-test_that("model_lists only carry one copy of training data", {
-  expect_s3_class(r_models[[1]]$trainingData, "data.frame")
-  expect_s3_class(c_models[[1]]$trainingData, "data.frame")
-  expect_s3_class(double_model_as[[1]]$trainingData, "data.frame")
-  expect_s3_class(single_model_as[[1]]$trainingData, "data.frame")
-  expect_s3_class(single_model_tune[[1]]$trainingData, "data.frame")
-
+test_that("model_lists only carry training data in recipe", {
+  expect_null(r_models[[1]]$trainingData)
+  expect_null(c_models[[1]]$trainingData)
   expect_null(r_models[[2]]$trainingData)
-  expect_null(c_models[[2]]$trainingData)
-  expect_null(double_model_as[[2]]$trainingData)
+  expect_null(m_models[[1]]$trainingData)
 })
 
 test_that("[ extracts models by index", {
   expect_s3_class(r_flash[1], "model_list")
-  expect_s3_class(r_flash[2], "model_list")
-  expect_s3_class(r_flash[2:3], "model_list")
-  expect_equal(r_flash[seq_along(r_flash)], r_flash)
+  expect_s3_class(r_models[2], "model_list")
+  expect_s3_class(r_models[1:2], "model_list")
+  expect_equivalent(r_models[seq_along(r_models)], r_models)
 })
 
 test_that("[ extracts by name, index, or logical vector", {
-  expect_equal(double_model_as[1], double_model_as["Random Forest"])
-  expect_equal(c_models[1], c_models["Random Forest"])
-  expect_equal(double_model_as[1], double_model_as[c(TRUE, FALSE)])
-  expect_equal(double_model_as[2], double_model_as["eXtreme Gradient Boosting"])
-  expect_equal(double_model_as[1:2], double_model_as[c("Random Forest", "eXtreme Gradient Boosting")])
-  expect_equal(double_model_as, double_model_as[c(TRUE, TRUE)])
+  expect_equivalent(r_models[1], r_models[names(r_models)[1]])
+  expect_equivalent(c_models[1], c_models[names(c_models)[1]])
+  expect_equivalent(r_models[1], r_models[c(TRUE, FALSE)])
+  expect_equivalent(r_models[2], r_models[names(r_models)[2]])
+  expect_equivalent(r_models[1:2], r_models[names(r_models)[1:2]])
+  expect_equivalent(r_models, r_models[c(TRUE, TRUE)])
 })
 
 test_that("metrics and predict are same for extracted best model", {
   best <- extract_model_info(c_models)$best_model_name
   extracted <- c_models[best]
-  expect_equal(evaluate(c_models), evaluate(extracted))
-  expect_equal(predict(c_models), predict(extracted))
+  expect_equivalent(evaluate(c_models), evaluate(extracted))
+  expect_equivalent(predict(c_models), predict(extracted))
 })
 
 test_that("performance drops if the best model is pulled out of model_list", {
